@@ -37,7 +37,6 @@
 #include <time.h>
 #include <libgnome/gnome-i18n.h>
 #include <string.h>
-#include "nsBuildID.h"
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
 #include <nsIPrefService.h>
@@ -45,9 +44,8 @@
 #include <nsIServiceManager.h>
 #include <nsIIOService.h>
 #include <nsIProtocolProxyService.h>
-#include <nsIJVMManager.h>
 #include <nsIAtom.h>
-#include <nsIFontList.h>
+#include <nsIFontEnumerator.h>
 #include <nsISupportsPrimitives.h>
 #include <nsReadableUtils.h>
 #include <nsICookieManager.h>
@@ -57,6 +55,10 @@
 #include <nsCCookieManager.h>
 #include <nsCPasswordManager.h>
 
+// FIXME: For setting the locale. hopefully gtkmozembed will do itself soon
+#include <nsIChromeRegistry.h>
+#include <nsILocaleService.h>
+
 #define MOZILLA_PROFILE_DIR  "/mozilla"
 #define MOZILLA_PROFILE_NAME "epiphany"
 #define MOZILLA_PROFILE_FILE "prefs.js"
@@ -171,10 +173,14 @@ encodings[] =
 	{ N_("_Georgian (GEOSTD8)"),                "geostd8",               LG_OTHER },
 	{ N_("_Icelandic (MacIcelandic)"),          "x-mac-icelandic",       LG_OTHER },
 	{ N_("_Nordic (ISO-8859-10)"),              "ISO-8859-10",           LG_OTHER },
-	{ N_("Romanian (_MacRomanian)"),            "x-mac-romanian",        LG_OTHER },
+	{ N_("_Romanian (MacRomanian)"),            "x-mac-romanian",        LG_OTHER },
 	{ N_("R_omanian (ISO-8859-16)"),            "ISO-8859-16",           LG_OTHER },
-	{ N_("South European (_ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER },
-	{ N_("_Thai (TIS-620)"),                    "TIS-620",               LG_OTHER },
+	{ N_("South _European (ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER },
+	{ N_("Thai (TIS-_620)"),                    "TIS-620",               LG_OTHER },
+#if MOZILLA_SNAPSHOT >= 9
+	{ N_("Thai (IS_O-8859-11)"),                "iso-8859-11",           LG_OTHER },
+	{ N_("_Thai (Windows-874)"),                "windows-874",           LG_OTHER },
+#endif	
 	{ N_("_User Defined"),                      "x-user-defined",        LG_OTHER },
 };
 static const guint n_encodings = G_N_ELEMENTS (encodings);
@@ -246,8 +252,6 @@ struct MozillaEmbedSinglePrivate
 	GtkWidget *theme_window;
 };
 
-static NS_DEFINE_CID(kJVMManagerCID, NS_JVMMANAGER_CID);
-
 static GObjectClass *parent_class = NULL;
 
 GType
@@ -302,23 +306,29 @@ mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 	shell_class->show_file_picker = impl_show_file_picker;
 }
 
-static void
+EphyEmbedSingle *
+mozilla_embed_single_new (void)
+{
+	return EPHY_EMBED_SINGLE (g_object_new (MOZILLA_EMBED_SINGLE_TYPE, NULL));
+}
+
+static gboolean
 mozilla_set_default_prefs (MozillaEmbedSingle *mes)
 {
 	nsCOMPtr<nsIPrefService> prefService;
 
         prefService = do_GetService (NS_PREFSERVICE_CONTRACTID);
-	g_return_if_fail (prefService != NULL);
+	if (prefService == NULL) return FALSE;
 
         nsCOMPtr<nsIPrefBranch> pref;
         prefService->GetBranch ("", getter_AddRefs(pref));
-	g_return_if_fail (pref != NULL);
+	if (pref == NULL) return FALSE;
 
 	/* Don't allow mozilla to raise window when setting focus (work around bugs) */
 	pref->SetBoolPref ("mozilla.widget.raise-on-setfocus", PR_FALSE);
 
 	/* set default search engine */
-	pref->SetCharPref ("keyword.URL", "http://www.google.com/search?btnI=I%27m+Feeling+Lucky&q=");
+	pref->SetCharPref ("keyword.URL", "http://www.google.com/search?q=");
 	pref->SetBoolPref ("keyword.enabled", PR_TRUE);
 	pref->SetBoolPref ("security.checkloaduri", PR_FALSE);
 
@@ -345,9 +355,6 @@ mozilla_set_default_prefs (MozillaEmbedSingle *mes)
 	/* Make sure this is off, we monitor for theme changes and set colors. */
 	pref->SetBoolPref ("browser.display.use_system_colors", PR_FALSE);
 
-	/* Smooth scrolling on */
-	pref->SetBoolPref ("general.smoothScroll", PR_TRUE);
-
 	/* Disable blinking text and marquee, its non-standard and annoying */
 	pref->SetBoolPref ("browser.blink_allowed", PR_FALSE);
 	pref->SetBoolPref ("browser.display.enable_marquee", PR_FALSE);
@@ -370,15 +377,12 @@ mozilla_set_default_prefs (MozillaEmbedSingle *mes)
 	pref->SetIntPref ("mousewheel.withcontrolkey.numlines", 1);
 	pref->SetBoolPref ("mousewheel.withcontrolkey.sysnumlines", PR_FALSE);
 
-	/* Enable Image Auto-Resizing */
-	pref->SetBoolPref ("browser.enable_automatic_image_resizing", PR_TRUE);
-
-	/* User agent */
-
 	/* FIXME We need to do this because mozilla doesnt set product
 	sub for embedding apps */
 	pref->SetCharPref ("general.useragent.vendor", "Epiphany");
 	pref->SetCharPref ("general.useragent.vendorSub", VERSION);
+
+	return TRUE;
 }
 
 static char *
@@ -527,13 +531,71 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 {
  	mes->priv = g_new0 (MozillaEmbedSinglePrivate, 1);
 
+	mes->priv->theme_window = NULL;
 	mes->priv->user_prefs =
 		g_build_filename (ephy_dot_dir (), 
 				  MOZILLA_PROFILE_DIR,
 				  MOZILLA_PROFILE_NAME,
 				  MOZILLA_PROFILE_FILE,
 				  NULL);
+}
 
+static nsresult
+getUILang (nsAString& aUILang)
+{
+	nsresult result;
+
+	nsCOMPtr<nsILocaleService> localeService = do_GetService (NS_LOCALESERVICE_CONTRACTID, &result);
+	if (NS_FAILED (result) || !localeService)
+	{
+		g_warning ("Could not get locale service!\n");
+		return NS_ERROR_FAILURE;
+	}
+
+	nsXPIDLString uiLang;
+	result = localeService->GetLocaleComponentForUserAgent (getter_Copies(uiLang));
+	aUILang = uiLang;
+	if (NS_FAILED (result))
+	{
+		g_warning ("Could not determine locale!\n");
+		return NS_ERROR_FAILURE;
+	}
+
+	return NS_OK;
+}
+
+static nsresult
+mozilla_init_chrome (void)
+{
+	nsresult result;
+	nsAutoString uiLang;
+
+	nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService (NS_CHROMEREGISTRY_CONTRACTID, &result);
+	if (NS_FAILED (result) || !chromeRegistry)
+	{
+		g_warning ("Could not get the chrome registry!\n");
+		return NS_ERROR_FAILURE;
+	}
+
+	// Set skin to 'classic' so we get native scrollbars.
+	result = chromeRegistry->SelectSkin (NS_LITERAL_CSTRING("classic/1.0"), PR_FALSE);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	// set locale
+	chromeRegistry->SetRuntimeProvider(PR_TRUE);
+
+	result = getUILang(uiLang);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	result = chromeRegistry->SelectLocale (NS_ConvertUCS2toUTF8(uiLang), PR_FALSE);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	return NS_OK;
+}
+
+gboolean
+mozilla_embed_single_init_services (MozillaEmbedSingle *single)
+{
 	/* Pre initialization */
 	mozilla_init_home ();
 	mozilla_init_profile ();
@@ -541,24 +603,30 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 	/* Fire up the best */
 	gtk_moz_embed_push_startup ();
 
-	mozilla_set_default_prefs (mes);
+	/* Until gtkmozembed does this itself */
+	mozilla_init_chrome ();
+
+	mozilla_init_single (single);
+
+	if (!mozilla_set_default_prefs (single))
+	{
+		return FALSE;
+	}
 
 	/* FIXME: This should be removed when mozilla
 	 * bugs 207000 and 207001 are fixed.
 	 */
-	mozilla_setup_colors (mes);
+	mozilla_setup_colors (single);
 
 	START_PROFILER ("Mozilla prefs notifiers")
-	mozilla_notifiers_init (EPHY_EMBED_SINGLE (mes));
+	mozilla_notifiers_init (EPHY_EMBED_SINGLE (single));
 	STOP_PROFILER ("Mozilla prefs notifiers")
 
-	mozilla_init_single (mes);
-	
 	mozilla_register_components ();
 
 	mozilla_register_external_protocols ();
 
-	/* FIXME alert if fails */
+	return TRUE;
 }
 
 static void 
@@ -628,7 +696,10 @@ mozilla_embed_single_finalize (GObject *object)
 
 	g_free (mes->priv->user_prefs);
 
-	gtk_widget_destroy (mes->priv->theme_window);
+	if (mes->priv->theme_window)
+	{
+		gtk_widget_destroy (mes->priv->theme_window);
+	}
 	
         g_free (mes->priv);
 }
@@ -776,54 +847,34 @@ impl_get_font_list (EphyEmbedSingle *shell,
 		    char **default_font)
 {
 	nsresult rv;
-
-	nsCOMPtr<nsIFontList> mozFontList;
-	mozFontList = do_CreateInstance("@mozilla.org/gfx/fontlist;1", &rv);
-	if(NS_FAILED(rv)) return G_FAILED;
-
-	nsCOMPtr<nsISimpleEnumerator> fontEnum;
-	mozFontList->AvailableFonts(NS_ConvertUTF8toUCS2(langGroup).get(),
-				    NS_ConvertUTF8toUCS2(fontType).get(),
-				    getter_AddRefs(fontEnum));
-	if(NS_FAILED(rv)) return G_FAILED;
-
+	PRUint32 fontCount;
+	PRUnichar **fontArray;
 	GList *l = NULL;
-	PRBool enumResult;
-	for(fontEnum->HasMoreElements(&enumResult) ;
-	    enumResult == PR_TRUE;
-	    fontEnum->HasMoreElements(&enumResult))
+
+	nsCOMPtr<nsIFontEnumerator> mozFontEnumerator;
+	mozFontEnumerator = do_CreateInstance("@mozilla.org/gfx/fontenumerator;1", &rv);
+	if(NS_FAILED(rv)) return G_FAILED;
+
+	rv = mozFontEnumerator->EnumerateFonts (nsnull, nsnull,
+					        &fontCount, &fontArray);
+	if (NS_FAILED (rv)) return G_FAILED;
+
+	for (PRUint32 i = 0; i < fontCount; i++)
 	{
-		nsCOMPtr<nsISupportsString> fontName;
-		fontEnum->GetNext(getter_AddRefs(fontName));
-		if(NS_FAILED(rv)) return G_FAILED;
-
-		nsString fontString;
-		fontName->GetData(fontString);
-
 		char *gFontString;
-		gFontString = g_strdup(NS_ConvertUCS2toUTF8(fontString).get());
+
+		gFontString = g_strdup(NS_ConvertUCS2toUTF8 (fontArray[i]).get());
 		l = g_list_prepend (l, gFontString);
+		nsMemory::Free (fontArray[i]);
 	}
+
+	nsMemory::Free (fontArray);
+
 	*fontList = g_list_reverse (l);
 
 	if (default_font != NULL)
 	{
-		char key[255];
-		char *value = NULL;
-		nsCOMPtr<nsIPrefService> prefService;
-
-	        prefService = do_GetService (NS_PREFSERVICE_CONTRACTID);
-		g_return_val_if_fail (prefService != NULL, G_FAILED);
-	
-	        nsCOMPtr<nsIPrefBranch> pref;
-	        prefService->GetBranch ("", getter_AddRefs(pref));
-		g_return_val_if_fail (pref != NULL, G_FAILED);
-
-		sprintf (key, "font.name.%s.%s", fontType, langGroup);
-		
-		pref->GetCharPref (key, &value);
-		*default_font = g_strdup (value);
-		nsMemory::Free (value);
+		*default_font = g_strdup (fontType);
 	}
 
 	return G_OK;
