@@ -37,7 +37,6 @@
 #include "ephy-history-window.h"
 #include "ephy-shell.h"
 #include "ephy-dnd.h"
-#include "ephy-prefs.h"
 #include "egg-action-group.h"
 #include "egg-toggle-action.h"
 #include "egg-menu-merge.h"
@@ -49,13 +48,16 @@
 #include "ephy-stock-icons.h"
 #include "ephy-gui.h"
 #include "toolbar.h"
+#include "ephy-stock-icons.h"
+#include "ephy-search-entry.h"
+#include "session.h"
+#include "ephy-favicon-cache.h"
 
 static GtkTargetEntry page_drag_types [] =
 {
         { EPHY_DND_URI_LIST_TYPE,   0, 0 },
         { EPHY_DND_TEXT_TYPE,       0, 1 },
-        { EPHY_DND_URL_TYPE,        0, 2 },
-	{ EPHY_DND_BOOKMARK_TYPE,   0, 3 }
+        { EPHY_DND_URL_TYPE,        0, 2 }
 };
 static int n_page_drag_types = G_N_ELEMENTS (page_drag_types);
 
@@ -72,8 +74,6 @@ static void ephy_history_window_get_property (GObject *object,
 					      GParamSpec *pspec);
 static void ephy_history_window_dispose      (GObject *object);
 
-static void search_entry_changed_cb       (GtkWidget *entry,
-					   EphyHistoryWindow *editor);
 static void cmd_open_bookmarks_in_tabs    (EggAction *action,
 					   EphyHistoryWindow *editor);
 static void cmd_open_bookmarks_in_browser (EggAction *action,
@@ -194,7 +194,7 @@ confirmation_dialog_response_cb (GtkDialog *dialog, gint response,
 	if (response != GTK_RESPONSE_OK)
 		return;
 
-	session = ephy_shell_get_session (ephy_shell);
+	session = SESSION (ephy_shell_get_session (ephy_shell));
 	windows = session_get_windows (session);
 
 	for (; windows != NULL; windows = windows->next)
@@ -216,21 +216,46 @@ confirmation_dialog_construct (EphyHistoryWindow *editor)
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *image;
+	GtkWidget *button;
+	GtkWidget *align;
 	char *str;
 
 	dialog = gtk_dialog_new_with_buttons (_("Clear history"),
 					     GTK_WINDOW (editor),
 					     GTK_DIALOG_DESTROY_WITH_PARENT |
 					     GTK_DIALOG_NO_SEPARATOR,
-					     GTK_STOCK_CANCEL,
-					     GTK_RESPONSE_CANCEL,
-					     GTK_STOCK_CLEAR,
-					     GTK_RESPONSE_OK,
 					     NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
 
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+	gtk_widget_show (button);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_CANCEL);
+
+	button = gtk_button_new ();
+	gtk_widget_show (button);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_widget_show (align);
+	gtk_container_add (GTK_CONTAINER (button), align);
+
+	hbox = gtk_hbox_new (FALSE, 2);
+	gtk_widget_show (hbox);
+	gtk_container_add (GTK_CONTAINER (align), hbox);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+
+	label = gtk_label_new_with_mnemonic (_("C_lear"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_widget_show (hbox);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
@@ -321,14 +346,14 @@ cmd_open_bookmarks_in_tabs (EggAction *action,
 
 	for (l = selection; l; l = l->next)
 	{
-		EphyNode *node = EPHY_NODE (l->data);
+		EphyNode *node = l->data;
 		const char *location;
 
 		location = ephy_node_get_property_string (node,
 						EPHY_NODE_PAGE_PROP_LOCATION);
 
 		ephy_shell_new_tab (ephy_shell, window, NULL, location,
-			EPHY_NEW_TAB_APPEND_GROUPED | EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+			EPHY_NEW_TAB_OPEN_PAGE | EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 	}
 
 	g_list_free (selection);
@@ -347,13 +372,14 @@ cmd_open_bookmarks_in_browser (EggAction *action,
 
 	for (l = selection; l; l = l->next)
 	{
-		EphyNode *node = EPHY_NODE (l->data);
+		EphyNode *node = l->data;
 		const char *location;
 
 		location = ephy_node_get_property_string (node,
 						EPHY_NODE_PAGE_PROP_LOCATION);
 
 		ephy_shell_new_tab (ephy_shell, window, NULL, location,
+				    EPHY_NEW_TAB_OPEN_PAGE |
 				    EPHY_NEW_TAB_IN_NEW_WINDOW);
 	}
 
@@ -392,7 +418,7 @@ cmd_copy (EggAction *action,
 		if (g_list_length (selection) == 1)
 		{
 			const char *tmp;
-			EphyNode *node = EPHY_NODE (selection->data);
+			EphyNode *node = selection->data;
 			tmp = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_LOCATION);
 			gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), tmp, -1);
 		}
@@ -463,7 +489,7 @@ cmd_bookmark_link (EggAction *action,
 		const char *title;
 		EphyNode *node;
 
-		node = EPHY_NODE (selection->data);
+		node = selection->data;
 		location = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_LOCATION);
 		title = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_TITLE);
 		if (ephy_new_bookmark_is_unique (bookmarks, GTK_WINDOW (window),
@@ -577,23 +603,14 @@ ephy_history_window_node_activated_cb (GtkWidget *view,
 	const char *location;
 	EphyWindow *window;
 
-	g_return_if_fail (EPHY_IS_NODE (node));
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_PAGE_PROP_LOCATION);
 	g_return_if_fail (location != NULL);
 
 	window = EPHY_WINDOW (get_target_window (editor));
-	if (window != NULL)
-	{
-		ephy_window_load_url (EPHY_WINDOW (window), location);
-		gtk_window_present (GTK_WINDOW (window));
-	}
-	else
-	{
-		/* We have to create a browser window */
-		ephy_shell_new_tab (ephy_shell, NULL, NULL, location,
-				    EPHY_NEW_TAB_IN_NEW_WINDOW);
-	}
+	
+	ephy_shell_new_tab (ephy_shell, NULL, NULL, location,
+			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
 static void
@@ -683,7 +700,7 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 	g_object_set (action, "sensitive", select_all, NULL);
 	action = egg_action_group_get_action (action_group, "Delete");
 	g_object_set (action, "sensitive", delete, NULL);
-	action = egg_action_group_get_action (action_group, "BookmarkPage");
+	action = egg_action_group_get_action (action_group, "BookmarkLink");
 	g_object_set (action, "sensitive", bookmark_page, NULL);
 }
 
@@ -780,20 +797,6 @@ key_pressed_cb (EphyNodeView *view,
 	return FALSE;
 }
 
-static void
-reset_search_entry (EphyHistoryWindow *editor)
-{
-	g_signal_handlers_block_by_func
-		(G_OBJECT (editor->priv->search_entry),
-		 G_CALLBACK (search_entry_changed_cb),
-		 editor);
-	gtk_entry_set_text (GTK_ENTRY (editor->priv->search_entry), "");
-	g_signal_handlers_unblock_by_func
-		(G_OBJECT (editor->priv->search_entry),
-		 G_CALLBACK (search_entry_changed_cb),
-		 editor);
-}
-
 static void
 site_node_selected_cb (EphyNodeView *view,
 		       EphyNode *node,
@@ -808,16 +811,15 @@ site_node_selected_cb (EphyNodeView *view,
 	}
 	else
 	{
-		reset_search_entry (editor);
+		ephy_search_entry_clear (EPHY_SEARCH_ENTRY (editor->priv->search_entry));
 		pages_filter (editor, node);
 	}
 }
 
 static void
-search_entry_changed_cb (GtkWidget *entry, EphyHistoryWindow *editor)
+search_entry_search_cb (GtkWidget *entry, char *search_text, EphyHistoryWindow *editor)
 {
 	EphyNode *all;
-	char *search_text;
 
 	g_signal_handlers_block_by_func
 		(G_OBJECT (editor->priv->sites_view),
@@ -831,8 +833,6 @@ search_entry_changed_cb (GtkWidget *entry, EphyHistoryWindow *editor)
 		 G_CALLBACK (site_node_selected_cb),
 		 editor);
 
-	search_text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-
 	GDK_THREADS_ENTER ();
 
 	ephy_node_filter_empty (editor->priv->pages_filter);
@@ -849,8 +849,6 @@ search_entry_changed_cb (GtkWidget *entry, EphyHistoryWindow *editor)
 	ephy_node_filter_done_changing (editor->priv->pages_filter);
 
 	GDK_THREADS_LEAVE ();
-
-	g_free (search_text);
 }
 
 static GtkWidget *
@@ -865,13 +863,13 @@ build_search_box (EphyHistoryWindow *editor)
 	gtk_container_set_border_width (GTK_CONTAINER (box), 6);
 	gtk_widget_show (box);
 
-	entry = gtk_entry_new ();
+	entry = ephy_search_entry_new ();
 	add_focus_monitor (editor, entry);
 	add_entry_monitor (editor, entry);
 	editor->priv->search_entry = entry;
 	gtk_widget_show (entry);
-	g_signal_connect (G_OBJECT (entry), "changed",
-			  G_CALLBACK (search_entry_changed_cb),
+	g_signal_connect (G_OBJECT (entry), "search",
+			  G_CALLBACK (search_entry_search_cb),
 			  editor);
 
 	label = gtk_label_new (NULL);
@@ -913,7 +911,8 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	const char *icon_location;
 	GdkPixbuf *pixbuf = NULL;
 
-	cache = ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell));
+	cache = EPHY_FAVICON_CACHE
+		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
 	icon_location = ephy_node_get_property_string
 		(node, EPHY_NODE_PAGE_PROP_ICON);
 
@@ -925,7 +924,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	}
 
 	g_value_init (value, GDK_TYPE_PIXBUF);
-	g_value_set_object (value, pixbuf);
+	g_value_set_object_take_ownership (value, pixbuf);
 }
 
 static void
@@ -946,13 +945,16 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	EggMenuMerge *ui_merge;
 	EggActionGroup *action_group;
 	EggAction *action;
-	const char *icon_path;
-	int i;
+	GdkPixbuf *icon;
+	int i, col_id;
 
 	gtk_window_set_title (GTK_WINDOW (editor), _("History"));
 
-	icon_path = ephy_file ("epiphany-history.png");
-	gtk_window_set_icon_from_file (GTK_WINDOW (editor), icon_path, NULL);
+	icon = gtk_widget_render_icon (GTK_WIDGET (editor), 
+				       EPHY_STOCK_HISTORY,
+				       GTK_ICON_SIZE_MENU,
+				       NULL);
+	gtk_window_set_icon (GTK_WINDOW(editor), icon);
 
 	g_signal_connect (editor, "delete_event",
 			  G_CALLBACK (delete_event_cb), NULL);
@@ -1005,19 +1007,23 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	gtk_widget_show (scrolled_window);
 	sites_view = ephy_node_view_new (node, NULL);
 	add_focus_monitor (editor, sites_view);
+	col_id = ephy_node_view_add_data_column (EPHY_NODE_VIEW (sites_view),
+					         G_TYPE_STRING,
+					         EPHY_NODE_PAGE_PROP_LOCATION,
+						 NULL, NULL);
 	ephy_node_view_select_node (EPHY_NODE_VIEW (sites_view),
 				    ephy_history_get_pages (editor->priv->history));
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (sites_view),
 					   page_drag_types,
-				           n_page_drag_types,
-					   EPHY_NODE_PAGE_PROP_LOCATION);
+				           n_page_drag_types, col_id);
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sites_view));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 	ephy_node_view_add_column (EPHY_NODE_VIEW (sites_view), _("Sites"),
 				   G_TYPE_STRING,
 				   EPHY_NODE_PAGE_PROP_TITLE,
 				   EPHY_NODE_PAGE_PROP_PRIORITY,
-				   EPHY_NODE_VIEW_AUTO_SORT,
+				   EPHY_NODE_VIEW_AUTO_SORT |
+				   EPHY_NODE_VIEW_SEARCHABLE,
 				   provide_favicon);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), sites_view);
 	gtk_widget_show (sites_view);
@@ -1055,13 +1061,17 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	add_focus_monitor (editor, pages_view);
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pages_view));
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (pages_view), TRUE);
+	col_id = ephy_node_view_add_data_column (EPHY_NODE_VIEW (pages_view),
+					         G_TYPE_STRING,
+					         EPHY_NODE_PAGE_PROP_LOCATION,
+						 NULL, NULL);
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (pages_view),
 					   page_drag_types,
-				           n_page_drag_types,
-					   EPHY_NODE_PAGE_PROP_LOCATION);
+				           n_page_drag_types, col_id);
 	col = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Title"),
 				         G_TYPE_STRING, EPHY_NODE_PAGE_PROP_TITLE,
-				         -1, EPHY_NODE_VIEW_USER_SORT, NULL);
+				         -1, EPHY_NODE_VIEW_USER_SORT |
+					 EPHY_NODE_VIEW_SEARCHABLE, NULL);
 	gtk_tree_view_column_set_max_width (col, 250);
 	col = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Address"),
 				         G_TYPE_STRING, EPHY_NODE_PAGE_PROP_LOCATION,
