@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.97 2003/07/13 08:52:35 mpeseng Exp $
+ *  $Id: ephy-window.c,v 1.100 2003/08/05 16:40:20 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -331,6 +331,17 @@ ephy_window_get_type (void)
         return ephy_window_type;
 }
 
+static void
+remove_from_session (EphyWindow *window)
+{
+	Session *session;
+
+	session = SESSION (ephy_shell_get_session (ephy_shell));
+	g_return_if_fail (session != NULL);
+
+	session_remove_window (session, window);
+}
+
 static void
 ephy_window_destroy (GtkObject *gtkobject)
 {
@@ -338,7 +349,12 @@ ephy_window_destroy (GtkObject *gtkobject)
 
 	LOG ("EphyWindow destroy %p", window)
 
-	window->priv->closing = TRUE;
+	if (window->priv->closing == FALSE)
+	{
+		window->priv->closing = TRUE;
+
+		remove_from_session (window);
+	}
 
 	if (window->priv->exit_fullscreen_popup)
 	{
@@ -702,32 +718,6 @@ setup_window (EphyWindow *window)
 			  window);
 }
 
-static void
-sync_find_dialog (FindDialog *dialog, GParamSpec *spec, EphyWindow *window)
-{
-	EggActionGroup *action_group;
-	EggAction *action;
-	FindNavigationFlags flags;
-	gboolean can_go_prev = FALSE, can_go_next = FALSE;
-
-	flags = find_dialog_get_navigation_flags (dialog);
-
-	if (flags & FIND_CAN_GO_PREV)
-	{
-		can_go_prev = TRUE;
-	}
-	if (flags & FIND_CAN_GO_NEXT)
-	{
-		can_go_next = TRUE;
-	}
-
-	action_group = window->priv->action_group;
-	action = egg_action_group_get_action (action_group, "EditFindPrev");
-	g_object_set (action, "sensitive", can_go_prev, NULL);
-	action = egg_action_group_get_action (action_group, "EditFindNext");
-	g_object_set (action, "sensitive", can_go_next, NULL);
-}
-
 static void
 sync_tab_address (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 {
@@ -1087,10 +1077,10 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	{
 		popup = "EphyImagePopup";
 	}
-	else if (context & EMBED_CONTEXT_INPUT)
+/*	else if (context & EMBED_CONTEXT_INPUT)
 	{
 		popup = "EphyInputPopup";
-	}
+	}*/
 	else
 	{
 		popup = framed ? "EphyFramedDocumentPopup" :
@@ -1460,17 +1450,6 @@ save_window_chrome (EphyWindow *window)
 	}
 }
 
-static void
-remove_from_session (EphyWindow *window)
-{
-	Session *session;
-
-	session = SESSION (ephy_shell_get_session (ephy_shell));
-	g_return_if_fail (session != NULL);
-
-	session_remove_window (session, window);
-}
-
 static void
 ephy_window_finalize (GObject *object)
 {
@@ -1482,8 +1461,6 @@ ephy_window_finalize (GObject *object)
 
         g_return_if_fail (window->priv != NULL);
 
-	remove_from_session (window);
-
 	if (window->priv->find_dialog)
 	{
 		g_object_unref (G_OBJECT (window->priv->find_dialog));
@@ -1855,31 +1832,26 @@ ephy_window_notebook_switch_page_cb (GtkNotebook *notebook,
 	update_tabs_menu_sensitivity (window);
 }
 
-EphyDialog *
-ephy_window_get_find_dialog (EphyWindow *window)
+void
+ephy_window_find (EphyWindow *window)
 {
 	EphyDialog *dialog;
 	EphyEmbed *embed;
 
-	if (window->priv->find_dialog)
+	if (window->priv->find_dialog == NULL)
 	{
-		return window->priv->find_dialog;
+		embed = ephy_window_get_active_embed (window);
+		g_return_if_fail (GTK_IS_WINDOW(window));
+
+		dialog = find_dialog_new_with_parent (GTK_WIDGET(window),
+						      embed);
+		window->priv->find_dialog = dialog;
+
+		g_object_add_weak_pointer(G_OBJECT (dialog),
+					  (gpointer *) &window->priv->find_dialog);
 	}
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_val_if_fail (GTK_IS_WINDOW(window), NULL);
-	dialog = find_dialog_new_with_parent (GTK_WIDGET(window),
-					      embed);
-
-	sync_find_dialog (FIND_DIALOG (dialog), NULL, window);
-	g_signal_connect_object (dialog, "notify::embed",
-				 G_CALLBACK (sync_find_dialog), window, 0);
-	g_signal_connect_object (dialog, "notify::navigation",
-				 G_CALLBACK (sync_find_dialog), window, 0);
-
-	window->priv->find_dialog = dialog;
-
-	return dialog;
+	ephy_dialog_show (window->priv->find_dialog);
 }
 
 static void
